package Core::SMART;

use strict;
use warnings;

use base 'Resmon::Module';

use Resmon::ExtComm qw(run_command cache_command);

=pod

=head1 NAME

Core::SMART - pulls SMART values from disk drives

=head1 SYNOPSIS

 Core::SMART {
     sda : smartctl_cmd => /usr/sbin/smartctl , smartctl_args => -d sat\,12
 }

=head1 DESCRIPTION

Self-Monitoring, Analysis and Reporting Technology (SMART) allows for detecting
errors and reporting on reliability indicators on hard disk drives.

This module pulls the specified values from the output of the smartctl utility.
For more information on smartctl, see http://smartmontools.sourceforge.net/

=head1 CONFIGURATION

=over

=item check_name

This indicates the disk argument to be given to smartctl.  Expects the basename
of the device, not the full path, e.g. "sda" not "/dev/sda".
Check names having "-" and a nubmer after it are considered to be physical parts of 
megaraid arrays.

=item smartctl_cmd

The full path of the smartctl command.  Defaults to /usr/sbin/smartctl.

=item smartctl_args

Optional list of additional arguments that will be passed to smartctl.  
The default arguments are "-i -A".  Any arguments specified here will be 
added to this list.

If the argument contains a comma, it must be escaped with a backslash.

=back

=head1 METRICS

The SMART attribute data is highly vendor-specific.  Each attribute will produce
two metrics, one with the normalized value and one with the raw value.

Post-processing may be desirable in specific cases, such as when the value as
hex has special meaning.

=over

=item model

Device model as reported in the information section.

=item serial

Device serial number as reported in the information section.

=item fw

Device firmware version as reported in the information section.

=item (attribute)

The normalized value of the attribute, expected to be an integer.

=item (attribute)_raw

The raw value of the attribute, expected to be an integer.

=back

=cut

sub handler {
    my $self = shift;
    my $config = $self->{config};
    my ($disk_name,$megaraid_number) = split('-',$self->{check_name}),
    my $disk;
    $disk = "/dev/$disk_name" if ($^O eq "linux") || ($^O =~ /bsd/) ;
    $disk = "/dev/rdsk/$disk_name" if $^O eq "solaris";
    my $smartctl_cmd = $config->{smartctl_cmd} || "/usr/sbin/smartctl";
    my $smartctl_args = "-i -a"; 
    $smartctl_args .= " -H -d megaraid,$megaraid_number" if  (defined ($megaraid_number));
    $smartctl_args = "$smartctl_args $config->{smartctl_args}" if $config->{smartctl_args};
    my $smartdata = {};
    my $output = run_command("$smartctl_cmd $smartctl_args $disk");

    foreach my $line (split(/\n/, $output)) {
        if ( ($line =~ /^Device Model:\s+(.+)$/)
           ||($line =~ /^Product:\s+(.+)$/) )
        {
            $smartdata->{"model"} = [$1, "s"];
        }
        elsif ($line =~ /^Serial Number:\s+(.+)$/i) {
            $smartdata->{"serial"} = [$1, "s"];
        }
        elsif ($line =~ /^SMART Health Status:\s+(.+)$/i) {
            $smartdata->{"health"} = [$1, "s"];
        }
        elsif ( ($line =~ /^Firmware Version:\s+(.+)$/)
              ||($line =~ /^Revision:\s+(.+)$/) )
        {
            $smartdata->{"firmware_version"} = [$1, "s"];
        }
        elsif ($line =~ /^Elements in grown defect list:\s+(.+)$/) {
            $smartdata->{"elements_in_defect_list"} = [$1, "i"];
        }
        elsif ($line =~ /^(verify):\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\S+)\s+(\d+)$/) {
            print STDERR "$1 $1_errors_corrected $5 $1_errors_uncorrected $8\n";
            $smartdata->{"$1_errors_corrected"} = [$5, "i"];
            $smartdata->{"$1_errors_uncorrected"} = [$8, "i"];
        }
        elsif ($line =~ /^(read|write):\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\S+)\s+(\d+)$/) {
            print STDERR "$1 $1_errors_corrected $5 $1_errors_uncorrected $8\n";
            $smartdata->{"$1_errors_corrected"} = [$5, "i"];
            $smartdata->{"$1_errors_uncorrected"} = [$8, "i"];
            $smartdata->{"verify_errors_corrected"} =   [0, "i"] unless defined($smartdata->{"verify_errors_corrected"});
            $smartdata->{"verify_errors_uncorrected"} = [0, "i"] unless defined($smartdata->{"verify_errors_uncorrected"});
        }
        elsif ($line =~ /^Non-medium error count:\s+(.+)$/) {
            $smartdata->{"non_medium_errors"} = [$1, "i"];
        }
	# Capture id, attribute_name, threshold, fail, raw_value (columns 1,2,6,7,8)
=pod
Vendor Specific SMART Attributes with Thresholds:
ID# ATTRIBUTE_NAME          FLAG     VALUE WORST THRESH TYPE      UPDATED  WHEN_FAILED RAW_VALUE
  1 Raw_Read_Error_Rate     0x000b   100   100   016    Pre-fail  Always       -       0
  2 Throughput_Performance  0x0004   137   137   000    Old_age   Offline      -       91
  3 Spin_Up_Time            0x0007   120   120   024    Pre-fail  Always       -       317 (Average 313)

Per https://www.backblaze.com/blog/what-smart-stats-indicate-hard-drive-failures/,
we should check attributes:
ATTRIBUTE		DESCRIPTION
SMART 5			Reallocated Sectors Count
SMART 187		Reported Uncorrectable Errors
SMART 188		Command Timeout
SMART 197		Current Pending Sector Count
SMART 198		Uncorrectable Sector Count
=cut
        elsif ( ( $line =~ /^\s*(5|187|188|197|198)\s/ )
              && ($line =~ /^\s*(\d+)\s(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+\S+\s+(\S+)\s+(\S+)/) 
              ) {
            $smartdata->{"$1_name"} = [$2, "s"];
            $smartdata->{"$1_thresh"} = [$3, "i"];
            $smartdata->{"$1_whenfailed"} = [$4, "s"];
            $smartdata->{"$1_raw"} = [$5, "i"];
        }
    }

    return $smartdata;
};
1;
